/* ------------------------------------------------------------------------ */
/*  HOS-V4  kernel configuration                                            */
/*    kernel object create and initialize                                   */
/*                                                                          */
/* ------------------------------------------------------------------------ */


#include "kernel.h"
#include "kernel_id.h"

#include "netclock3.h"
#include "ostimer.h"
#include "system.h"
#include "itu.h"
#include "comm.h"



/* ------------------------------------------ */
/*                 idle stack                 */
/* ------------------------------------------ */

VP         mknl_idl_stkblk[((256) + sizeof(VP) - 1) / sizeof(VP)];	/* idle stack block*/
const VP   mknl_idl_stk   = (VP)(mknl_idl_stkblk);	/* idle stack */
const SIZE mknl_idl_stksz = (SIZE)sizeof(mknl_idl_stkblk);	/* idle stack size */



/* ------------------------------------------ */
/*           create ready queue               */
/* ------------------------------------------ */

T_MKNL_QUE mknl_rdq_tbl[8];
const INT  mknl_rdq_cnt = 8;



/* ------------------------------------------ */
/*             interrupt stack                */
/* ------------------------------------------ */

const VP kernel_int_sp = (VP)(0xFFFF20);	/* interrupt stack pointer */



/* ------------------------------------------ */
/*               set time tic                 */
/* ------------------------------------------ */

const UW kernel_tic_deno = TIC_DENO;
const UW kernel_tic_div  = TIC_NUME / TIC_DENO;
const UW kernel_tic_mod  = TIC_NUME % TIC_DENO;



/* ------------------------------------------ */
/*          create task objects               */
/* ------------------------------------------ */

/* stack area */
static VP kernel_tsk1_stk[(((((((676 + 958 + 256 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk2_stk[(((((((134 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk3_stk[(((((((684 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk4_stk[(((((((184 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk5_stk[(((((((444 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk6_stk[(((((((190 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk7_stk[(((((((190 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk8_stk[(((((((662 + 1536 + 256 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk9_stk[(((((((662 + 1536 + 256 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk10_stk[(((((((662 + 1536 + 256 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];
static VP kernel_tsk11_stk[(((((((662 + 1536 + 256 + 32) - 1) / 256) + 1) * 256) * 2) + sizeof(VP) - 1) / sizeof(VP)];

/* task control block for rom area */
const T_KERNEL_TCB_ROM kernel_tcb_rom[11] =
	{
		{(ATR)(TA_HLNG), (VP_INT)(1), (FP)(Terminal), (PRI)(6), (SIZE)((((((676 + 958 + 256 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk1_stk},
		{(ATR)(TA_HLNG), (VP_INT)(1), (FP)(SCI_Snd), (PRI)(8), (SIZE)((((((134 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk2_stk},
		{(ATR)(TA_HLNG), (VP_INT)(0), (FP)(Idle), (PRI)(8), (SIZE)((((((684 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk3_stk},
		{(ATR)(TA_HLNG), (VP_INT)((0x00200000UL)), (FP)(RtlRecIvent), (PRI)(3), (SIZE)((((((184 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk4_stk},
		{(ATR)(TA_HLNG), (VP_INT)(0), (FP)(LinkLayerRecieve), (PRI)(4), (SIZE)((((((444 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk5_stk},
		{(ATR)(TA_HLNG), (VP_INT)((0x00200000UL)), (FP)(RtlSendIvent), (PRI)(3), (SIZE)((((((190 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk6_stk},
		{(ATR)(TA_HLNG), (VP_INT)((0x00200000UL)), (FP)(RtlOverWrite), (PRI)(2), (SIZE)((((((190 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk7_stk},
		{(ATR)(TA_HLNG), (VP_INT)(1), (FP)(httpd), (PRI)(7), (SIZE)((((((662 + 1536 + 256 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk8_stk},
		{(ATR)(TA_HLNG), (VP_INT)(1), (FP)(httpd), (PRI)(7), (SIZE)((((((662 + 1536 + 256 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk9_stk},
		{(ATR)(TA_HLNG), (VP_INT)(1), (FP)(httpd), (PRI)(7), (SIZE)((((((662 + 1536 + 256 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk10_stk},
		{(ATR)(TA_HLNG), (VP_INT)(1), (FP)(httpd), (PRI)(7), (SIZE)((((((662 + 1536 + 256 + 32) - 1) / 256) + 1) * 256) * 2), (VP)kernel_tsk11_stk},
	};

/* task control block for ram area */
T_KERNEL_TCB_RAM kernel_tcb_ram[11];

/* task control block table */
T_KERNEL_TCB_RAM *kernel_tcb_ram_tbl[11] =
	{
		&kernel_tcb_ram[0],
		&kernel_tcb_ram[1],
		&kernel_tcb_ram[2],
		&kernel_tcb_ram[3],
		&kernel_tcb_ram[4],
		&kernel_tcb_ram[5],
		&kernel_tcb_ram[6],
		&kernel_tcb_ram[7],
		&kernel_tcb_ram[8],
		&kernel_tcb_ram[9],
		&kernel_tcb_ram[10],
	};

/* task control block count */
const INT kernel_tcb_cnt = 11;



/* ------------------------------------------ */
/*         create semaphore objects           */
/* ------------------------------------------ */

/* semaphore control block for rom area */
const T_KERNEL_SEMCB_ROM kernel_semcb_rom[3] =
	{
		{(ATR)(TA_TFIFO), (UINT)(MBX_NUMBER), (UINT)(MBX_NUMBER)},
		{(ATR)(TA_TFIFO), (UINT)(1), (UINT)(1)},
		{(ATR)(TA_TFIFO), (UINT)(1), (UINT)(1)},
	};

/* semaphore control block for ram area */
T_KERNEL_SEMCB_RAM kernel_semcb_ram[3];

/* semaphore control block table */
T_KERNEL_SEMCB_RAM *kernel_semcb_ram_tbl[3] =
	{
		&kernel_semcb_ram[0],
		&kernel_semcb_ram[1],
		&kernel_semcb_ram[2],
	};

/* semaphore control block count */
const INT kernel_semcb_cnt = 3;



/* ------------------------------------------ */
/*        create event flag objects           */
/* ------------------------------------------ */

/* event flag control block for rom area */
const T_KERNEL_FLGCB_ROM kernel_flgcb_rom[3] =
	{
		{(ATR)(TA_TFIFO | TA_WSGL), (FLGPTN)(0)},
		{(ATR)(TA_TFIFO | TA_WSGL), (FLGPTN)(0)},
		{(ATR)(TA_TFIFO | TA_WSGL), (FLGPTN)(0)},
	};

/* event flag control block for ram area */
T_KERNEL_FLGCB_RAM kernel_flgcb_ram[3];

/* event flag control block table */
T_KERNEL_FLGCB_RAM *kernel_flgcb_ram_tbl[16] =
	{
		&kernel_flgcb_ram[0],
		&kernel_flgcb_ram[1],
		&kernel_flgcb_ram[2],
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
	};

/* event flag control block count */
const INT kernel_flgcb_cnt = 16;



/* ------------------------------------------ */
/*        create data queue objects           */
/* ------------------------------------------ */

/* data queue control block count */
const INT kernel_dtqcb_cnt = 0;



/* ------------------------------------------ */
/*         create mail box objects            */
/* ------------------------------------------ */

/* message priority que header area */
static UB kernel_mbx1_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx2_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx3_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx4_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx5_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx6_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx7_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx8_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx9_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx10_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx11_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx12_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx13_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx14_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx15_mprihd[TSZ_MPRIHD(1)];
static UB kernel_mbx16_mprihd[TSZ_MPRIHD(1)];

/* mail box control block for rom area */
const T_KERNEL_MBXCB_ROM kernel_mbxcb_rom[16] =
	{
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx1_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx2_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx3_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx4_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx5_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx6_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx7_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx8_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx9_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx10_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx11_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx12_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx13_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx14_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx15_mprihd},
		{(ATR)(TA_MFIFO), (PRI)(1), (VP)kernel_mbx16_mprihd},
	};

/* mail box control block for ram area */
T_KERNEL_MBXCB_RAM kernel_mbxcb_ram[16];

/* mail box control block table */
T_KERNEL_MBXCB_RAM *kernel_mbxcb_ram_tbl[16] =
	{
		&kernel_mbxcb_ram[0],
		&kernel_mbxcb_ram[1],
		&kernel_mbxcb_ram[2],
		&kernel_mbxcb_ram[3],
		&kernel_mbxcb_ram[4],
		&kernel_mbxcb_ram[5],
		&kernel_mbxcb_ram[6],
		&kernel_mbxcb_ram[7],
		&kernel_mbxcb_ram[8],
		&kernel_mbxcb_ram[9],
		&kernel_mbxcb_ram[10],
		&kernel_mbxcb_ram[11],
		&kernel_mbxcb_ram[12],
		&kernel_mbxcb_ram[13],
		&kernel_mbxcb_ram[14],
		&kernel_mbxcb_ram[15],
	};

/* mail box control block count */
const INT kernel_mbxcb_cnt = 16;



/* ------------------------------------------ */
/*      create message buffer objects         */
/* ------------------------------------------ */

/* message buffer area */
static VP kernel_mbf1[(32)];

/* message buffer control block for rom area */
const T_KERNEL_MBFCB_ROM kernel_mbfcb_rom[1] =
	{
		{(ATR)(TA_TFIFO), (UINT)(4), (SIZE)(32),(VP)kernel_mbf1},
	};

/* message buffer control block for ram area */
T_KERNEL_MBFCB_RAM kernel_mbfcb_ram[1];

/* message buffer control block table */
T_KERNEL_MBFCB_RAM *kernel_mbfcb_ram_tbl[1] =
	{
		&kernel_mbfcb_ram[0],
	};

/* mail box control block count */
const INT kernel_mbfcb_cnt = 1;



/* ------------------------------------------ */
/*   create fixed size memory-pool objects    */
/* ------------------------------------------ */

/* fixed size memory-pool area */
static VP kernel_mpf1_heap[LARGE_BLOCKS][(((LARGE_BLOCK_SIZE) + sizeof(VP) - 1) / sizeof(VP))];
static VP kernel_mpf2_heap[MIDDLE_BLOCKS][(((MIDDLE_BLOCK_SIZE) + sizeof(VP) - 1) / sizeof(VP))];
static VP kernel_mpf3_heap[SMALL_BLOCKS][(((SMALL_BLOCK_SIZE) + sizeof(VP) - 1) / sizeof(VP))];
static VP kernel_mpf4_heap[VERY_SMALL_BLOCKS][(((VERY_SMALL_BLOCK_SIZE) + sizeof(VP) - 1) / sizeof(VP))];
static VP kernel_mpf5_heap[SCI_BLOCK_SIZE][(((SERIAL_BUFFER_SIZE) + sizeof(VP) - 1) / sizeof(VP))];
static VP kernel_mpf6_heap[16][(((128) + sizeof(VP) - 1) / sizeof(VP))];
static VP kernel_mpf7_heap[8][(((256) + sizeof(VP) - 1) / sizeof(VP))];
static VP kernel_mpf8_heap[4][(((640) + sizeof(VP) - 1) / sizeof(VP))];
static VP kernel_mpf9_heap[8][(((1536) + sizeof(VP) - 1) / sizeof(VP))];
static VP kernel_mpf10_heap[1][(((2048) + sizeof(VP) - 1) / sizeof(VP))];

/* fixed size memory-pool control block for rom area */
const T_KERNEL_MPFCB_ROM kernel_mpfcb_rom[10] =
	{
		{(ATR)(TA_TFIFO), (UINT)(LARGE_BLOCKS), (SIZE)sizeof(kernel_mpf1_heap[0]), (VP)kernel_mpf1_heap},
		{(ATR)(TA_TFIFO), (UINT)(MIDDLE_BLOCKS), (SIZE)sizeof(kernel_mpf2_heap[0]), (VP)kernel_mpf2_heap},
		{(ATR)(TA_TFIFO), (UINT)(SMALL_BLOCKS), (SIZE)sizeof(kernel_mpf3_heap[0]), (VP)kernel_mpf3_heap},
		{(ATR)(TA_TFIFO), (UINT)(VERY_SMALL_BLOCKS), (SIZE)sizeof(kernel_mpf4_heap[0]), (VP)kernel_mpf4_heap},
		{(ATR)(TA_TFIFO), (UINT)(SCI_BLOCK_SIZE), (SIZE)sizeof(kernel_mpf5_heap[0]), (VP)kernel_mpf5_heap},
		{(ATR)(TA_TFIFO), (UINT)(16), (SIZE)sizeof(kernel_mpf6_heap[0]), (VP)kernel_mpf6_heap},
		{(ATR)(TA_TFIFO), (UINT)(8), (SIZE)sizeof(kernel_mpf7_heap[0]), (VP)kernel_mpf7_heap},
		{(ATR)(TA_TFIFO), (UINT)(4), (SIZE)sizeof(kernel_mpf8_heap[0]), (VP)kernel_mpf8_heap},
		{(ATR)(TA_TFIFO), (UINT)(8), (SIZE)sizeof(kernel_mpf9_heap[0]), (VP)kernel_mpf9_heap},
		{(ATR)(TA_TFIFO), (UINT)(1), (SIZE)sizeof(kernel_mpf10_heap[0]), (VP)kernel_mpf10_heap},
	};

/* fixed size memory-pool control block for ram area */
T_KERNEL_MPFCB_RAM kernel_mpfcb_ram[10];

/* fixed size memory-pool control block table */
T_KERNEL_MPFCB_RAM *kernel_mpfcb_ram_tbl[10] =
	{
		&kernel_mpfcb_ram[0],
		&kernel_mpfcb_ram[1],
		&kernel_mpfcb_ram[2],
		&kernel_mpfcb_ram[3],
		&kernel_mpfcb_ram[4],
		&kernel_mpfcb_ram[5],
		&kernel_mpfcb_ram[6],
		&kernel_mpfcb_ram[7],
		&kernel_mpfcb_ram[8],
		&kernel_mpfcb_ram[9],
	};

/* fixed size memory-pool control block count */
const INT kernel_mpfcb_cnt = 10;



/* ------------------------------------------ */
/*  create variable size memory-pool objects  */
/* ------------------------------------------ */

/* variable size memory-pool control block count */
const INT kernel_mplcb_cnt = 0;



/* ------------------------------------------ */
/*       create cyclic handler objects        */
/* ------------------------------------------ */

/* cyclic handler control block for rom area */
const T_KERNEL_CYCCB_ROM kernel_cyccb_rom[1] =
	{
		{(ATR)(TA_HLNG), (VP_INT)((0x00200000UL)), (FP)(ETH0Trap), (RELTIM)(4)},
	};

/* cyclic handler control block for ram area */
T_KERNEL_CYCCB_RAM kernel_cyccb_ram[1];

/* cyclic handler control block table */
T_KERNEL_CYCCB_RAM *kernel_cyccb_ram_tbl[1] =
	{
		&kernel_cyccb_ram[0],
	};

/* cyclic handler control block count */
const INT kernel_cyccb_cnt = 1;



/* ------------------------------------------ */
/*       create alarm handler objects         */
/* ------------------------------------------ */

/* alarm handler control block count */
const INT kernel_almcb_cnt = 0;



/* ------------------------------------------ */
/*        interrupt control objects           */
/* ------------------------------------------ */

/* interrupt control */
T_KERNEL_INTCB kernel_intcb_tbl[64];		/* interrupt control block table */
const INT      kernel_intcb_cnt = 64;		/* interrupt control block count */
const INTNO    kernel_min_intno = 0;		/* minimum intrrupt number */

/* interrupt service routine control */
const INT      kernel_isrcb_cnt = 0;		/* ISR control block count */



/* ------------------------------------------ */
/*    CPU exception handler control objects   */
/* ------------------------------------------ */

/* interrupt control */
T_KERNEL_EXCCB kernel_exccb_tbl[1];		/* CPU exception handler control block table */
const INT      kernel_exccb_cnt = 1;		/* CPU exception handler control block count */
const EXCNO    kernel_min_excno = 0;		/* minimum CPU exception handler number */



/* ------------------------------------------ */
/*          initialize functions              */
/* ------------------------------------------ */

/* object initialize */
void kernel_cfg_init(void)
{
	int i;
	
	
	/* initialize task control block */
	for ( i = 0; i < 11; i++ )
	{
		kernel_tcb_ram[i].tcb_rom = &kernel_tcb_rom[i];
	}
	
	
	/* initialize semaphore control block */
	for ( i = 0; i < 3; i++ )
	{
		kernel_semcb_ram[i].semcb_rom = &kernel_semcb_rom[i];
	}
	
	
	/* initialize event flag control block */
	for ( i = 0; i < 3; i++ )
	{
		kernel_flgcb_ram[i].flgcb_rom = &kernel_flgcb_rom[i];
	}
	
	
	/* initialize mail box control block */
	for ( i = 0; i < 16; i++ )
	{
		kernel_mbxcb_ram[i].mbxcb_rom = &kernel_mbxcb_rom[i];
	}
	
	
	/* initialize message buffer control block */
	for ( i = 0; i < 1; i++ )
	{
		kernel_mbfcb_ram[i].mbfcb_rom = &kernel_mbfcb_rom[i];
	}
	
	
	/* initialize fixed size memory-pool control block */
	for ( i = 0; i < 10; i++ )
	{
		kernel_mpfcb_ram[i].mpfcb_rom = &kernel_mpfcb_rom[i];
	}
	
	
	/* initialize cyclic handler control block */
	for ( i = 0; i < 1; i++ )
	{
		kernel_cyccb_ram[i].cyccb_rom = &kernel_cyccb_rom[i];
	}

	/* initialize interrupt table */
	kernel_intcb_tbl[24].exinf = (VP_INT)(0);
	kernel_intcb_tbl[24].isr   = (FP)(ostim_hdr);
	kernel_intcb_tbl[56].exinf = (VP_INT)(1);
	kernel_intcb_tbl[56].isr   = (FP)(SCI_Err);
	kernel_intcb_tbl[57].exinf = (VP_INT)(1);
	kernel_intcb_tbl[57].isr   = (FP)(Int_SCI_Recv);
}

/* start up */
void kernel_cfg_start(void)
{
	kernel_ini_tsk();		/* initialize task */
	kernel_ini_sem();		/* initialize semaphore */
	kernel_ini_flg();		/* initialize event flag */
	kernel_ini_mbx();		/* initialize mail box */
	kernel_ini_mbf();		/* initialize message buffer */
	kernel_ini_mpf();		/* initialize fixed size memory-pool */
	kernel_ini_cyc();		/* initialize cyclic handler */

	/* call initialize routine*/
	((FP)(ostim_init))((VP_INT)(TIM_1m));
	((FP)(SampleInitialize))((VP_INT)(0));
}


/* ------------------------------------------------------------------------ */
/*  End of file                                                             */
/* ------------------------------------------------------------------------ */
